@@ -36,22 +36,21 @@ try:
 except ImportError:
     from urllib.parse import urlparse, urlsplit, urljoin  # NOQA
 
+from blinker import signal
 try:
     import pyphen
 except ImportError:
     pyphen = None
 
-import lxml.html
-from yapsy.PluginManager import PluginManager
-import pytz
-
+import logging
 if os.getenv('NIKOLA_DEBUG'):
-    import logging
     logging.basicConfig(level=logging.DEBUG)
 else:
-    import logging
     logging.basicConfig(level=logging.ERROR)
 
+import lxml.html
+from yapsy.PluginManager import PluginManager
+
 from .post import Post
 from . import utils
 from .plugin_categories import (
@@ -62,8 +61,10 @@ from .plugin_categories import (
     Task,
     TaskMultiplier,
     TemplateSystem,
+    SignalHandler,
 )
 
+
 config_changed = utils.config_changed
 
 __all__ = ['Nikola']
@@ -85,8 +86,15 @@ class Nikola(object):
     def __init__(self, **config):
         """Setup proper environment for running tasks."""
 
+        # Register our own path handlers
+        self.path_handlers = {
+            'slug': self.slug_path,
+            'post_path': self.post_path,
+        }
+
         self.strict = False
         self.global_data = {}
+        self.posts = []
         self.posts_per_year = defaultdict(list)
         self.posts_per_month = defaultdict(list)
         self.posts_per_tag = defaultdict(list)
@@ -97,6 +105,7 @@ class Nikola(object):
         self._scanned = False
         self._template_system = None
         self._THEMES = None
+        self.loghandlers = []
         if not config:
             self.configured = False
         else:
@@ -129,6 +138,7 @@ class Nikola(object):
             'CONTENT_FOOTER': '',
             'COPY_SOURCES': True,
             'CREATE_MONTHLY_ARCHIVE': False,
+            'CREATE_SINGLE_ARCHIVE': False,
             'DATE_FORMAT': '%Y-%m-%d %H:%M',
             'DEFAULT_LANG': "en",
             'DEPLOY_COMMANDS': [],
@@ -146,7 +156,7 @@ class Nikola(object):
             'GALLERY_SORT_BY_DATE': True,
             'GZIP_COMMAND': None,
             'GZIP_FILES': False,
-            'GZIP_EXTENSIONS': ('.txt', '.htm', '.html', '.css', '.js', '.json'),
+            'GZIP_EXTENSIONS': ('.txt', '.htm', '.html', '.css', '.js', '.json', '.xml'),
             'HIDE_SOURCELINK': False,
             'HIDE_UNTRANSLATED_POSTS': False,
             'HYPHENATE': False,
@@ -196,7 +206,8 @@ class Nikola(object):
             'DEPLOY_FUTURE': False,
             'SCHEDULE_ALL': False,
             'SCHEDULE_RULE': '',
-            'SCHEDULE_FORCE_TODAY': False
+            'SCHEDULE_FORCE_TODAY': False,
+            'LOGGING_HANDLERS': {'stderr': {'loglevel': 'WARNING', 'bubble': True}},
         }
 
         self.config.update(config)
@@ -301,6 +312,13 @@ class Nikola(object):
         self.default_lang = self.config['DEFAULT_LANG']
         self.translations = self.config['TRANSLATIONS']
 
+        locale_fallback, locale_default, locales = sanitized_locales(
+                                    self.config.get('LOCALE_FALLBACK', None),
+                                    self.config.get('LOCALE_DEFAULT', None),
+                                    self.config.get('LOCALES', {}),
+                                    self.translations)  # NOQA
+        utils.LocaleBorg.initialize(locales, self.default_lang)
+
         # BASE_URL defaults to SITE_URL
         if 'BASE_URL' not in self.config:
             self.config['BASE_URL'] = self.config.get('SITE_URL')
@@ -316,6 +334,7 @@ class Nikola(object):
             "PageCompiler": PageCompiler,
             "TaskMultiplier": TaskMultiplier,
             "RestExtension": RestExtension,
+            "SignalHandler": SignalHandler,
         })
         self.plugin_manager.setPluginInfoExtension('plugin')
         if sys.version_info[0] == 3:
@@ -331,6 +350,17 @@ class Nikola(object):
         self.plugin_manager.setPluginPlaces(places)
         self.plugin_manager.collectPlugins()
 
+        # Activate all required SignalHandler plugins
+        for plugin_info in self.plugin_manager.getPluginsOfCategory("SignalHandler"):
+            if plugin_info.name in self.config.get('DISABLED_PLUGINS'):
+                self.plugin_manager.removePluginFromCategory(plugin_info, "SignalHandler")
+            else:
+                self.plugin_manager.activatePluginByName(plugin_info.name)
+                plugin_info.plugin_object.set_site(self)
+
+        # Emit signal for SignalHandlers which need to start running immediately.
+        signal('sighandlers_loaded').send(self)
+
         self.commands = {}
         # Activate all command plugins
         for plugin_info in self.plugin_manager.getPluginsOfCategory("Command"):
@@ -376,7 +406,7 @@ class Nikola(object):
         self._GLOBAL_CONTEXT = {}
 
         self._GLOBAL_CONTEXT['_link'] = self.link
-        self._GLOBAL_CONTEXT['set_locale'] = s_l
+        self._GLOBAL_CONTEXT['set_locale'] = utils.LocaleBorg().set_locale
         self._GLOBAL_CONTEXT['rel_link'] = self.rel_link
         self._GLOBAL_CONTEXT['abs_link'] = self.abs_link
         self._GLOBAL_CONTEXT['exists'] = self.file_exists
@@ -440,6 +470,7 @@ class Nikola(object):
                 "PageCompiler"):
             self.compilers[plugin_info.name] = \
                 plugin_info.plugin_object
+        signal('configured').send(self)
 
     def _get_themes(self):
         if self._THEMES is None:
@@ -625,23 +656,14 @@ class Nikola(object):
         with open(output_name, "wb+") as post_file:
             post_file.write(data)
 
-    def current_lang(self):  # FIXME: this is duplicated, turn into a mixin
-        """Return the currently set locale, if it's one of the
-        available translations, or default_lang."""
-        lang = utils.LocaleBorg().current_lang
-        if lang:
-            if lang in self.translations:
-                return lang
-            lang = lang.split('_')[0]
-            if lang in self.translations:
-                return lang
-        # whatever
-        return self.default_lang
-
     def path(self, kind, name, lang=None, is_link=False):
         """Build the path to a certain kind of page.
 
-        kind is one of:
+        These are mostly defined by plugins by registering via
+        the register_path_handler method, except for slug and
+        post_path which are defined in this class' init method.
+
+        Here's some of the others, for historical reasons:
 
         * tag_index (name is ignored)
         * tag (and name is the tag name)
@@ -667,80 +689,9 @@ class Nikola(object):
         """
 
         if lang is None:
-            lang = self.current_lang()
+            lang = utils.LocaleBorg().current_lang
 
-        path = []
-
-        if kind == "tag_index":
-            path = [_f for _f in [self.config['TRANSLATIONS'][lang],
-                                  self.config['TAG_PATH'],
-                                  self.config['INDEX_FILE']] if _f]
-        elif kind == "tag":
-            if self.config['SLUG_TAG_PATH']:
-                name = utils.slugify(name)
-            path = [_f for _f in [self.config['TRANSLATIONS'][lang],
-                                  self.config['TAG_PATH'], name + ".html"] if
-                    _f]
-
-        elif kind == "category":
-            if self.config['SLUG_TAG_PATH']:
-                name = utils.slugify(name)
-            path = [_f for _f in [self.config['TRANSLATIONS'][lang],
-                                  self.config['TAG_PATH'], "cat_" + name + ".html"] if
-                    _f]
-        elif kind == "tag_rss":
-            if self.config['SLUG_TAG_PATH']:
-                name = utils.slugify(name)
-            path = [_f for _f in [self.config['TRANSLATIONS'][lang],
-                                  self.config['TAG_PATH'], name + ".xml"] if
-                    _f]
-        elif kind == "category_rss":
-            if self.config['SLUG_TAG_PATH']:
-                name = utils.slugify(name)
-            path = [_f for _f in [self.config['TRANSLATIONS'][lang],
-                                  self.config['TAG_PATH'], "cat_" + name + ".xml"] if
-                    _f]
-        elif kind == "index":
-            if name not in [None, 0]:
-                path = [_f for _f in [self.config['TRANSLATIONS'][lang],
-                                      self.config['INDEX_PATH'],
-                                      'index-{0}.html'.format(name)] if _f]
-            else:
-                path = [_f for _f in [self.config['TRANSLATIONS'][lang],
-                                      self.config['INDEX_PATH'],
-                                      self.config['INDEX_FILE']]
-                        if _f]
-        elif kind == "post_path":
-            path = [_f for _f in [self.config['TRANSLATIONS'][lang],
-                                  os.path.dirname(name),
-                                  self.config['INDEX_FILE']] if _f]
-        elif kind == "rss":
-            path = [_f for _f in [self.config['TRANSLATIONS'][lang],
-                                  self.config['RSS_PATH'], 'rss.xml'] if _f]
-        elif kind == "archive":
-            if name:
-                path = [_f for _f in [self.config['TRANSLATIONS'][lang],
-                                      self.config['ARCHIVE_PATH'], name,
-                                      self.config['INDEX_FILE']] if _f]
-            else:
-                path = [_f for _f in [self.config['TRANSLATIONS'][lang],
-                                      self.config['ARCHIVE_PATH'],
-                                      self.config['ARCHIVE_FILENAME']] if _f]
-        elif kind == "gallery":
-            path = [_f for _f in [self.config['GALLERY_PATH'], name,
-                                  self.config['INDEX_FILE']] if _f]
-        elif kind == "listing":
-            path = [_f for _f in [self.config['LISTINGS_FOLDER'], name +
-                                  '.html'] if _f]
-        elif kind == "slug":
-            results = [p for p in self.timeline if p.meta('slug') == name]
-            if not results:
-                utils.LOGGER.warning("Can't resolve path request for slug: {0}".format(name))
-            else:
-                if len(results) > 1:
-                    utils.LOGGER.warning('Ambiguous path request for slug: {0}'.format(name))
-                path = [_f for _f in results[0].permalink(lang).split('/') if _f]
-                utils.LOGGER.notice(path)
+        path = self.path_handlers[kind](name, lang)
 
         if is_link:
             link = '/' + ('/'.join(path))
@@ -753,6 +704,28 @@ class Nikola(object):
         else:
             return os.path.join(*path)
 
+    def post_path(self, name, lang):
+        """post_path path handler"""
+        return [_f for _f in [self.config['TRANSLATIONS'][lang],
+                              os.path.dirname(name),
+                              self.config['INDEX_FILE']] if _f]
+
+    def slug_path(self, name, lang):
+        """slug path handler"""
+        results = [p for p in self.timeline if p.meta('slug') == name]
+        if not results:
+            utils.LOGGER.warning("Can't resolve path request for slug: {0}".format(name))
+        else:
+            if len(results) > 1:
+                utils.LOGGER.warning('Ambiguous path request for slug: {0}'.format(name))
+            return [_f for _f in results[0].permalink(lang).split('/') if _f]
+
+    def register_path_handler(self, kind, f):
+        if kind in self.path_handlers:
+            utils.LOGGER.warning('Conflicting path handlers for kind: {0}'.format(kind))
+        else:
+            self.path_handlers[kind] = f
+
     def link(self, *args):
         return self.path(*args, is_link=True)
 
@@ -840,14 +813,6 @@ class Nikola(object):
             return
         seen = set([])
         print("Scanning posts", end='', file=sys.stderr)
-        tzinfo = None
-        if self.config['TIMEZONE'] is not None:
-            tzinfo = pytz.timezone(self.config['TIMEZONE'])
-        if self.config['FUTURE_IS_NOW']:
-            current_time = None
-        else:
-            current_time = utils.current_time(tzinfo)
-        targets = set([])
         lower_case_tags = set([])
         for wildcard, destination, template_name, use_in_feeds in \
                 self.config['post_pages']:
@@ -879,52 +844,33 @@ class Nikola(object):
                         seen.add(base_path)
                     post = Post(
                         base_path,
-                        self.config['CACHE_FOLDER'],
+                        self.config,
                         dest_dir,
                         use_in_feeds,
-                        self.config['TRANSLATIONS'],
-                        self.config['DEFAULT_LANG'],
-                        self.config['BASE_URL'],
                         self.MESSAGES,
                         template_name,
-                        self.config['FILE_METADATA_REGEXP'],
-                        self.config['STRIP_INDEXES'],
-                        self.config['INDEX_FILE'],
-                        tzinfo,
-                        current_time,
-                        self.config['HIDE_UNTRANSLATED_POSTS'],
-                        self.config['PRETTY_URLS'],
-                        self.config['HYPHENATE'],
+                        self.get_compiler(base_path).compile_html
                     )
-                    for lang, langpath in list(
-                            self.config['TRANSLATIONS'].items()):
-                        dest = (destination, langpath, dir_glob,
-                                post.meta[lang]['slug'])
-                        if dest in targets:
-                            raise Exception('Duplicated output path {0!r} '
-                                            'in post {1!r}'.format(
-                                                post.meta[lang]['slug'],
-                                                base_path))
-                        targets.add(dest)
-                    self.global_data[post.post_name] = post
+                    self.global_data[post.source_path] = post
                     if post.use_in_feeds:
+                        self.posts.append(post.source_path)
                         self.posts_per_year[
-                            str(post.date.year)].append(post.post_name)
+                            str(post.date.year)].append(post.source_path)
                         self.posts_per_month[
-                            '{0}/{1:02d}'.format(post.date.year, post.date.month)].append(post.post_name)
+                            '{0}/{1:02d}'.format(post.date.year, post.date.month)].append(post.source_path)
                         for tag in post.alltags:
                             if tag.lower() in lower_case_tags:
                                 if tag not in self.posts_per_tag:
                                     # Tags that differ only in case
                                     other_tag = [k for k in self.posts_per_tag.keys() if k.lower() == tag.lower()][0]
                                     utils.LOGGER.error('You have cases that differ only in upper/lower case: {0} and {1}'.format(tag, other_tag))
-                                    utils.LOGGER.error('Tag {0} is used in: {1}'.format(tag, post.post_name))
+                                    utils.LOGGER.error('Tag {0} is used in: {1}'.format(tag, post.source_path))
                                     utils.LOGGER.error('Tag {0} is used in: {1}'.format(other_tag, ', '.join(self.posts_per_tag[other_tag])))
                                     sys.exit(1)
                             else:
                                 lower_case_tags.add(tag.lower())
-                            self.posts_per_tag[tag].append(post.post_name)
-                        self.posts_per_category[post.meta('category')].append(post.post_name)
+                            self.posts_per_tag[tag].append(post.source_path)
+                        self.posts_per_category[post.meta('category')].append(post.source_path)
                     else:
                         self.pages.append(post)
                     if self.config['OLD_THEME_SUPPORT']:
@@ -1021,16 +967,155 @@ class Nikola(object):
         return utils.apply_filters(task, filters)
 
 
-def s_l(lang):
-    """A set_locale that uses utf8 encoding and returns ''."""
-    utils.LocaleBorg().current_lang = lang
+def sanitized_locales(locale_fallback, locale_default, locales, translations):
+    """Sanitizes all locales availble into a nikola session
+
+    There will be one locale for each language in translations.
+
+    Locales for languages not in translations are ignored.
+
+    An explicit locale for a language can be specified in locales[language].
+
+    Locales at the input must be in the string style (like 'en', 'en.utf8'), and
+    the string can be unicode or bytes; at the output will be of type str, as
+    required by locale.setlocale.
+
+    Explicit but invalid locales are replaced with the sanitized locale_fallback
+
+    Languages with no explicit locale are set to
+        the sanitized locale_default if it was explicitly set
+        sanitized guesses compatible with v 6.0.4 if locale_default was None
+
+    NOTE: never use locale.getlocale() , it can return values that
+    locale.setlocale will not accept in Windows XP, 7 and pythons 2.6, 2.7, 3.3
+    Examples: "Spanish", "French" can't do the full circle set / get / set
+    """
+    # locales for languages not in translations are ignored
+    extras = set(locales) - set(translations)
+    if extras:
+        msg = 'Unexpected languages in LOCALES, ignoring them: {0}'
+        utils.LOGGER.warn(msg.format(', '.join(extras)))
+        for lang in extras:
+            del locales[lang]
+
+    # py2x: get/setlocale related functions require the locale string as a str
+    # so convert
+    locale_fallback = str(locale_fallback) if locale_fallback else None
+    locale_default = str(locale_default) if locale_default else None
+    for lang in locales:
+        locales[lang] = str(locales[lang])
+
+    locale_fallback = valid_locale_fallback(locale_fallback)
+
+    # explicit but invalid locales are replaced with the sanitized locale_fallback
+    for lang in locales:
+        if not is_valid_locale(locales[lang]):
+            msg = 'Locale {0} for language {1} not accepted by python locale.'
+            utils.LOGGER.warn(msg.format(locales[lang], lang))
+            locales[lang] = locale_fallback
+
+    # languages with no explicit locale
+    missing = set(translations) - set(locales)
+    if locale_default:
+        # are set to the sanitized locale_default if it was explicitly set
+        if not is_valid_locale(locale_default):
+            msg = 'LOCALE_DEFAULT {0} could not be set, using {1}'
+            utils.LOGGER.warn(msg.format(locale_default, locale_fallback))
+            locale_default = locale_fallback
+        for lang in missing:
+            locales[lang] = locale_default
+    else:
+        # are set to sanitized guesses compatible with v 6.0.4 in Linux-Mac (was broken in Windows)
+        if sys.platform == 'win32':
+            guess_locale_fom_lang = guess_locale_from_lang_windows
+        else:
+            guess_locale_fom_lang = guess_locale_from_lang_posix
+        for lang in missing:
+            locale_n = guess_locale_fom_lang(lang)
+            if not locale:
+                locale_n = locale_fallback
+                msg = "Could not guess locale for language {0}, using locale {1}"
+                utils.LOGGER.warn(msg.format(lang, locale_n))
+            locales[lang] = locale_n
+
+    return locale_fallback, locale_default, locales
+
+
+def is_valid_locale(locale_n):
+    """True if locale_n is acceptable for locale.setlocale
+
+    for py2x compat locale_n should be of type str
+    """
     try:
-        locale.setlocale(locale.LC_ALL, (lang, "utf8"))
-    except Exception:
-        utils.LOGGER.warn(
-            "Could not set locale to {0}."
-            "This may cause some i18n features not to work.".format(lang))
-    return ''
+        locale.setlocale(locale.LC_ALL, locale_n)
+        return True
+    except locale.Error:
+        return False
+
+
+def valid_locale_fallback(desired_locale=None):
+    """returns a default fallback_locale, a string that locale.setlocale will accept
+
+    If desired_locale is provided must be of type str for py2x compatibility
+    """
+    # Whenever fallbacks change, adjust test TestHarcodedFallbacksWork
+    candidates_windows = [str('English'), str('C')]
+    candidates_posix = [str('en_US.utf8'), str('C')]
+    candidates = candidates_windows if sys.platform == 'win32' else candidates_posix
+    if desired_locale:
+        candidates = list(candidates)
+        candidates.insert(0, desired_locale)
+    found_valid = False
+    for locale_n in candidates:
+        found_valid = is_valid_locale(locale_n)
+        if found_valid:
+            break
+    if not found_valid:
+        msg = 'Could not find a valid fallback locale, tried: {0}'
+        utils.LOGGER.warn(msg.format(candidates))
+    elif desired_locale and (desired_locale != locale_n):
+        msg = 'Desired fallback locale {0} could not be set, using: {1}'
+        utils.LOGGER.warn(msg.format(desired_locale, locale_n))
+    return locale_n
+
+
+def guess_locale_from_lang_windows(lang):
+    return str(_windows_locale_guesses.get(lang, None))
+
+
+def guess_locale_from_lang_posix(lang):
+    # compatibility v6.0.4
+    if is_valid_locale(str(lang)):
+        locale_n = str(lang)
+    else:
+        # this works in Travis when locale support set by Travis suggestion
+        locale_n = str((locale.normalize(lang).split('.')[0]) + '.utf8')
+    return locale_n
+
+
+_windows_locale_guesses = {
+    # some languages may need that the appropiate Microsoft's Language Pack
+    # be instaled; the 'str' bit will be added in the guess function
+    "bg": "Bulgarian",
+    "ca": "Catalan",
+    "de": "German",
+    "el": "Greek",
+    "en": "English",
+    "eo": "Esperanto",
+    "es": "Spanish",
+    "fa": "Farsi",  # persian
+    "fr": "French",
+    "hr": "Croatian",
+    "it": "Italian",
+    "jp": "Japanese",
+    "nl": "Dutch",
+    "pl": "Polish",
+    "pt_br": "Portuguese_Brazil",
+    "ru": "Russian",
+    "sl_si": "Slovenian",
+    "tr_tr": "Turkish",
+    "zh_cn": "Chinese_China",  # Chinese (Simplified)
+}
 
 
 SOCIAL_BUTTONS_CODE = """
