@@ -33,7 +33,8 @@ __all__ = [
     'RestExtension',
     'Task',
     'TaskMultiplier',
-    'TemplateSystem'
+    'TemplateSystem',
+    'SignalHandler'
 ]
 
 from yapsy.IPlugin import IPlugin
@@ -198,6 +199,10 @@ class RestExtension(BasePlugin):
     name = "dummy_rest_extension"
 
 
+class SignalHandler(BasePlugin):
+    name = "dummy_signal_handler"
+
+
 class Importer(Command):
     """Basic structure for importing data into Nikola.
 
