@@ -24,10 +24,11 @@
 # OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 # SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-from __future__ import unicode_literals, print_function
+from __future__ import unicode_literals, print_function, absolute_import
 
 import codecs
 from collections import defaultdict
+import datetime
 import os
 import re
 import string
@@ -41,8 +42,20 @@ try:
     import pyphen
 except ImportError:
     pyphen = None
+import pytz
 
-from .utils import (to_datetime, slugify, bytes_str, Functionary, LocaleBorg, unicode_str)
+# for tearDown with _reload we cannot use 'from import' to get forLocaleBorg
+import nikola.utils
+from .utils import (
+    bytes_str,
+    current_time,
+    Functionary,
+    LOGGER,
+    slugify,
+    to_datetime,
+    unicode_str,
+)
+from .rc4 import rc4
 
 __all__ = ['Post']
 
@@ -55,67 +68,85 @@ class Post(object):
     """Represents a blog post or web page."""
 
     def __init__(
-        self, source_path, cache_folder, destination, use_in_feeds,
-        translations, default_lang, base_url, messages, template_name,
-        file_metadata_regexp=None, strip_indexes=False, index_file='index.html',
-        tzinfo=None, current_time=None, skip_untranslated=False, pretty_urls=False,
-        hyphenate=False,
+        self,
+        source_path,
+        config,
+        destination,
+        use_in_feeds,
+        messages,
+        template_name,
+        compiler
     ):
         """Initialize post.
 
-        The source path is the .txt post file. From it we calculate
+        The source path is the user created post file. From it we calculate
         the meta file, as well as any translations available, and
         the .html fragment file path.
         """
+        self.compiler = compiler
+        self.config = config
+        tzinfo = None
+        if self.config['TIMEZONE'] is not None:
+            tzinfo = pytz.timezone(self.config['TIMEZONE'])
+        if self.config['FUTURE_IS_NOW']:
+            self.current_time = None
+        else:
+            self.current_time = current_time(tzinfo)
         self.translated_to = set([])
         self._prev_post = None
         self._next_post = None
-        self.base_url = base_url
+        self.base_url = self.config['BASE_URL']
         self.is_draft = False
         self.is_retired = False
         self.is_mathjax = False
-        self.strip_indexes = strip_indexes
-        self.index_file = index_file
-        self.pretty_urls = pretty_urls
+        self.strip_indexes = self.config['STRIP_INDEXES']
+        self.index_file = self.config['INDEX_FILE']
+        self.pretty_urls = self.config['PRETTY_URLS']
         self.source_path = source_path  # posts/blah.txt
         self.post_name = os.path.splitext(source_path)[0]  # posts/blah
+        # cache[\/]posts[\/]blah.html
+        self.base_path = os.path.join(self.config['CACHE_FOLDER'], self.post_name + ".html")
         # cache/posts/blah.html
-        self.base_path = os.path.join(cache_folder, self.post_name + ".html")
+        self._base_path = self.base_path.replace('\\', '/')
         self.metadata_path = self.post_name + ".meta"  # posts/blah.meta
         self.folder = destination
-        self.translations = translations
-        self.default_lang = default_lang
+        self.translations = self.config['TRANSLATIONS']
+        self.default_lang = self.config['DEFAULT_LANG']
         self.messages = messages
-        self.skip_untranslated = skip_untranslated
+        self.skip_untranslated = self.config['HIDE_UNTRANSLATED_POSTS']
         self._template_name = template_name
         self.is_two_file = True
-        self.hyphenate = hyphenate
+        self.hyphenate = self.config['HYPHENATE']
         self._reading_time = None
 
-        default_metadata = get_meta(self, file_metadata_regexp)
+        default_metadata = get_meta(self, self.config['FILE_METADATA_REGEXP'])
 
         self.meta = Functionary(lambda: None, self.default_lang)
-        self.meta[default_lang] = default_metadata
+        self.meta[self.default_lang] = default_metadata
 
         # Load internationalized metadata
-        for lang in translations:
-            if lang != default_lang:
+        for lang in self.translations:
+            if lang != self.default_lang:
                 if os.path.isfile(self.source_path + "." + lang):
                     self.translated_to.add(lang)
 
                 meta = defaultdict(lambda: '')
                 meta.update(default_metadata)
-                meta.update(get_meta(self, file_metadata_regexp, lang))
+                meta.update(get_meta(self, self.config['FILE_METADATA_REGEXP'], lang))
                 self.meta[lang] = meta
             elif os.path.isfile(self.source_path):
-                self.translated_to.add(default_lang)
+                self.translated_to.add(self.default_lang)
 
-        if not self.is_translation_available(default_lang):
+        if not self.is_translation_available(self.default_lang):
             # Special case! (Issue #373)
             # Fill default_metadata with stuff from the other languages
             for lang in sorted(self.translated_to):
                 default_metadata.update(self.meta[lang])
 
+        if 'date' not in default_metadata and not use_in_feeds:
+            # For stories we don't *really* need a date
+            default_metadata['date'] = datetime.datetime.fromtimestamp(os.stat(self.source_path).st_ctime)
+
         if 'title' not in default_metadata or 'slug' not in default_metadata \
                 or 'date' not in default_metadata:
             raise OSError("You must set a title (found '{0}'), a slug (found "
@@ -126,9 +157,9 @@ class Post(object):
                                         source_path))
 
         # If timezone is set, build localized datetime.
-        self.date = to_datetime(self.meta[default_lang]['date'], tzinfo)
+        self.date = to_datetime(self.meta[self.default_lang]['date'], tzinfo)
 
-        self.publish_later = False if current_time is None else self.date >= current_time
+        self.publish_later = False if self.current_time is None else self.date >= self.current_time
 
         is_draft = False
         is_retired = False
@@ -173,7 +204,7 @@ class Post(object):
 
     @property
     def tags(self):
-        lang = self.current_lang()
+        lang = nikola.utils.LocaleBorg().current_lang
         if lang in self._tags:
             return self._tags[lang]
         elif self.default_lang in self._tags:
@@ -183,7 +214,7 @@ class Post(object):
 
     @property
     def prev_post(self):
-        lang = self.current_lang()
+        lang = nikola.utils.LocaleBorg().current_lang
         rv = self._prev_post
         while self.skip_untranslated:
             if rv is None:
@@ -199,7 +230,7 @@ class Post(object):
 
     @property
     def next_post(self):
-        lang = self.current_lang()
+        lang = nikola.utils.LocaleBorg().current_lang
         rv = self._next_post
         while self.skip_untranslated:
             if rv is None:
@@ -234,33 +265,20 @@ class Post(object):
             fmt_date = fmt_date.decode('utf8')
         return fmt_date
 
-    def current_lang(self):
-        """Return the currently set locale, if it's one of the
-        available translations, or default_lang."""
-        lang = LocaleBorg().current_lang
-        if lang:
-            if lang in self.translations:
-                return lang
-            lang = lang.split('_')[0]
-            if lang in self.translations:
-                return lang
-        # whatever
-        return self.default_lang
-
     def title(self, lang=None):
         """Return localized title.
 
-        If lang is not specified, it will use the currently set locale,
-        because templates set it.
+        If lang is not specified, it defaults to the current language from
+        templates, as set in LocaleBorg.
         """
         if lang is None:
-            lang = self.current_lang()
+            lang = nikola.utils.LocaleBorg().current_lang
         return self.meta[lang]['title']
 
     def description(self, lang=None):
         """Return localized description."""
         if lang is None:
-            lang = self.current_lang()
+            lang = nikola.utils.LocaleBorg().current_lang
         return self.meta[lang]['description']
 
     def deps(self, lang):
@@ -273,6 +291,32 @@ class Post(object):
         deps += self.fragment_deps(lang)
         return deps
 
+    def compile(self, lang):
+        """Generate the cache/ file with the compiled post."""
+
+        def wrap_encrypt(path, password):
+            """Wrap a post with encryption."""
+            with codecs.open(path, 'rb+', 'utf8') as inf:
+                data = inf.read() + "<!--tail-->"
+            data = CRYPT.substitute(data=rc4(password, data))
+            with codecs.open(path, 'wb+', 'utf8') as outf:
+                outf.write(data)
+
+        self.READ_MORE_LINK = self.config['READ_MORE_LINK']
+        dest = self.translated_base_path(lang)
+        if not self.is_translation_available(lang) and self.config['HIDE_UNTRANSLATED_POSTS']:
+            return
+        else:
+            self.compiler(
+                self.translated_source_path(lang),
+                dest,
+                self.is_two_file),
+        if self.meta('password'):
+            wrap_encrypt(dest, self.meta('password'))
+        if self.publish_later:
+            LOGGER.notice('{0} is scheduled to be published in the future ({1})'.format(
+                self.source_path, self.date))
+
     def fragment_deps(self, lang):
         """Return a list of dependencies to build this post's fragment."""
         deps = []
@@ -306,6 +350,13 @@ class Post(object):
         else:
             return '.'.join((self.source_path, sorted(self.translated_to)[0]))
 
+    def translated_base_path(self, lang):
+        """Return path to the translation's base_path file."""
+        if lang == self.default_lang:
+            return self.base_path
+        else:
+            return '.'.join((self.base_path, lang))
+
     def _translated_file_path(self, lang):
         """Return path to the translation's file, or to the original."""
         if lang in self.translated_to:
@@ -323,14 +374,14 @@ class Post(object):
 
         teaser_only=True breaks at the teaser marker and returns only the teaser.
         strip_html=True removes HTML tags
-        lang=None uses the currently set locale
+        lang=None uses the last used to set locale
 
         All links in the returned HTML will be relative.
         The HTML returned is a bare fragment, not a full document.
         """
 
         if lang is None:
-            lang = self.current_lang()
+            lang = nikola.utils.LocaleBorg().current_lang
         file_name = self._translated_file_path(lang)
         with codecs.open(file_name, "r", "utf8") as post_file:
             data = post_file.read().strip()
@@ -362,21 +413,26 @@ class Post(object):
         if teaser_only:
             teaser = TEASER_REGEXP.split(data)[0]
             if teaser != data:
-                if TEASER_REGEXP.search(data).groups()[-1]:
-                    teaser += '<p class="more"><a href="{0}">{1}</a></p>'.format(
-                        self.permalink(lang, absolute=really_absolute),
-                        TEASER_REGEXP.search(data).groups()[-1])
-                else:
-                    teaser += READ_MORE_LINK.format(
-                        link=self.permalink(lang, absolute=really_absolute),
-                        read_more=self.messages[lang]["Read more"])
+                if not strip_html:
+                    if TEASER_REGEXP.search(data).groups()[-1]:
+                        teaser += '<p class="more"><a href="{0}">{1}</a></p>'.format(
+                            self.permalink(lang, absolute=really_absolute),
+                            TEASER_REGEXP.search(data).groups()[-1])
+                    else:
+                        teaser += READ_MORE_LINK.format(
+                            link=self.permalink(lang, absolute=really_absolute),
+                            read_more=self.messages[lang]["Read more"])
                 # This closes all open tags and sanitizes the broken HTML
                 document = lxml.html.fromstring(teaser)
                 data = lxml.html.tostring(document, encoding='unicode')
 
         if data and strip_html:
-            content = lxml.html.fromstring(data)
-            data = content.text_content().strip()  # No whitespace wanted.
+            try:
+                # Not all posts have a body. For example, you may have a page statically defined in the template that does not take content as input.
+                content = lxml.html.fromstring(data)
+                data = content.text_content().strip()  # No whitespace wanted.
+            except lxml.etree.ParserError:
+                data = ""
         return data
 
     @property
@@ -404,7 +460,7 @@ class Post(object):
         Extension is used in the path if specified.
         """
         if lang is None:
-            lang = self.current_lang()
+            lang = nikola.utils.LocaleBorg().current_lang
         if self._has_pretty_url(lang):
             path = os.path.join(self.translations[lang],
                                 self.folder, self.meta[lang]['slug'], 'index' + extension)
@@ -417,7 +473,7 @@ class Post(object):
 
     def permalink(self, lang=None, absolute=False, extension='.html'):
         if lang is None:
-            lang = self.current_lang()
+            lang = nikola.utils.LocaleBorg().current_lang
 
         pieces = self.translations[lang].split(os.sep)
         pieces += self.folder.split(os.sep)
@@ -646,3 +702,53 @@ def insert_hyphens(node, hyphenator):
 
     for child in node.iterchildren():
         insert_hyphens(child, hyphenator)
+
+
+CRYPT = string.Template("""\
+<script>
+function rc4(key, str) {
+    var s = [], j = 0, x, res = '';
+    for (var i = 0; i < 256; i++) {
+        s[i] = i;
+    }
+    for (i = 0; i < 256; i++) {
+        j = (j + s[i] + key.charCodeAt(i % key.length)) % 256;
+        x = s[i];
+        s[i] = s[j];
+        s[j] = x;
+    }
+    i = 0;
+    j = 0;
+    for (var y = 0; y < str.length; y++) {
+        i = (i + 1) % 256;
+        j = (j + s[i]) % 256;
+        x = s[i];
+        s[i] = s[j];
+        s[j] = x;
+        res += String.fromCharCode(str.charCodeAt(y) ^ s[(s[i] + s[j]) % 256]);
+    }
+    return res;
+}
+function decrypt() {
+    key = $$("#key").val();
+    crypt_div = $$("#encr")
+    crypted = crypt_div.html();
+    decrypted = rc4(key, window.atob(crypted));
+    if (decrypted.substr(decrypted.length - 11) == "<!--tail-->"){
+        crypt_div.html(decrypted);
+        $$("#pwform").hide();
+        crypt_div.show();
+    } else { alert("Wrong password"); };
+}
+</script>
+
+<div id="encr" style="display: none;">${data}</div>
+<div id="pwform">
+<form onsubmit="javascript:decrypt(); return false;" class="form-inline">
+<fieldset>
+<legend>This post is password-protected.</legend>
+<input type="password" id="key" placeholder="Type password here">
+<button type="submit" class="btn">Show Content</button>
+</fieldset>
+</form>
+</div>""")
